// gulpプラグインの読み込み
const gulp = require('gulp');
const ejs = require('gulp-ejs'); //ejs
const rename = require('gulp-rename'); //ファイル出力時にファイル名を変える

const sass = require('gulp-dart-sass'); //sass
const postcss = require('gulp-postcss'); //生成されたcssを操作する
const autoprefixer = require('autoprefixer'); //自動でベンダープレフィックスを付与
const htmlbeautify = require("gulp-html-beautify"); //HTML生成後のコードを綺麗にする

const plumber = require("gulp-plumber"); //エラーによるタスクの強制停止を防止
const notify = require("gulp-notify"); //デスクトップ通知
const browserSync = require("browser-sync").create(); //変更を即座にブラウザへ反映

const fs = require('fs');//JSONファイル操作用
const del = require('del'); //データ削除用

// 入出力するフォルダを指定
const srcBase = './src';
const distBase = './dist';

const srcPath = {
  'scss': srcBase + '/scss/**/*.scss',
  'img': srcBase + '/img/**/*',
  'js': srcBase + '/js/**/*.js',
  'json': srcBase + '/**/*.json',
  'ejs': srcBase + '/**/*.ejs',
  '_ejs': '!' + srcBase + '/_inc/**/*.ejs',

};
const distPath = {
  'css': distBase + '/css',
  'html': distBase + '/**/*.html',
  'img': distBase + '/img',
  'js': distBase + '/js',
  'item': distBase + '/item',
};


/* clean */
const clean = () => {
  return del([distBase + '/**'], { force: true });
}


/* sass */
const cssSass = () => {
  return gulp.src(srcPath.scss, {
    sourcemaps: false
  })
    .pipe(
      //エラーが出ても処理を止めない
      plumber({
        errorHandler: notify.onError('Error:<%= error.message %>')
      }))
    .pipe(postcss([
      autoprefixer({
        browsers: ['last 2 versions', 'iOS >= 12', 'Android >= 8']
      })
    ]))
    .pipe(sass({ outputStyle: 'expanded' }))
    .pipe(gulp.dest(distPath.css)) //コンパイル先
    .pipe(browserSync.stream())
    .pipe(notify({
      message: 'Sassをコンパイルしました！',
      onLast: true
    }))
}

/* ejs */
const ejsFunc = () => {

  var jsonFile = srcBase + '/data/pages.json',
    json = JSON.parse(fs.readFileSync(jsonFile, 'utf8'));

  return gulp
    .src([srcPath.ejs, srcPath._ejs])
    .pipe(ejs({json:json}))
    .pipe(rename({
      basename: 'index', //ファイル名
      extname: '.html' //拡張子
    }))
    .pipe(gulp.dest(distBase))
}


/* image */
const imgFunc = () => {
  return gulp.src(srcPath.img)
    .pipe(gulp.dest(distPath.img))
}

/* js */
const jsFunc = () => {
  return gulp.src(srcPath.js)
    .pipe(gulp.dest(distPath.js))
}

/* ローカルサーバー立ち上げ */
const browserSyncFunc = () => {
  browserSync.init(browserSyncOption);
}

const browserSyncOption = {
  server: distBase
}

/* リロード */
const browserSyncReload = (done) => {
  browserSync.reload();
  done();
}

/* ファイルの変更時にbrowserSyncReloadする */
const watchFiles = () => {
  gulp.watch(srcPath.scss, gulp.series(cssSass))
  gulp.watch(srcPath.img, gulp.series(imgFunc, browserSyncReload))
  gulp.watch(srcPath.ejs, gulp.series(ejsFunc, browserSyncReload))
  gulp.watch(srcPath.js, gulp.series(jsFunc, browserSyncReload))
}

exports.default = gulp.series(
  clean,
  gulp.parallel(cssSass, ejsFunc, imgFunc, jsFunc),
  gulp.parallel(watchFiles, browserSyncFunc)
);

exports.build = gulp.series(
  clean,
  gulp.parallel(cssSass, ejsFunc, imgFunc, jsFunc),
);



